package guillermocs.patrimonio30

import android.content.ContentValues
import android.content.Context
import android.database.Cursor
import android.database.sqlite.SQLiteConstraintException
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteException
import android.database.sqlite.SQLiteOpenHelper
import android.util.Log
import guillermocs.patrimonio30.Clases.*
import android.text.TextUtils
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStream
import java.io.InputStreamReader


/**
 * Clase que administra la conexión de la base de datos y su estructuración
 */

class DBHelper(context: Context) : SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {

    companion object {
        // If you change the database schema, you must increment the database version.
        val DATABASE_VERSION = 1
        val DATABASE_NAME = "patrimonio.db"

        // Script sql para crear la estructura de la base de datos
        private val SQL_CREATE_ENTRIES_SITIO = String.format("create table %s (%s integer primary key autoincrement, %s text, %s text, %s text, %s text, %s text)",
                DBContract.Place.TABLE_NAME, DBContract.Place.ID, DBContract.Place.TITLE, DBContract.Place.IMAGE, DBContract.Place.DESCRIPTION,
                DBContract.Place.LOCATION, DBContract.Place.INTERESTTURL)

        private val SQL_CREATE_ENTRIES_TEST = String.format("create table %s (%s int primary key, %s text, %s text)",
                DBContract.Test.TABLE_NAME, DBContract.Test.ID, DBContract.Test.NAME, DBContract.Test.SCORE)

        private val SQL_CREATE_ENTRIES_PREGUNTA = String.format("create table %s (%s int primary key, %s text, %s text, %s text, %s text, %s text, %s text)",
                DBContract.Question.TABLE_NAME, DBContract.Question.ID, DBContract.Question.TEXT, DBContract.Question.ANSWER1,
                DBContract.Question.ANSWER2, DBContract.Question.ANSWER3, DBContract.Question.ANSWER4,
                DBContract.Question.CORRECTANSWER)

        private val SQL_CREATE_ENTRIES_STP = String.format("create table %s (%s int, %s int, %s int,primary key(%s,%s,%s))",
                DBContract.PlaceTestQuestion.TABLE_NAME,
                DBContract.PlaceTestQuestion.IDPLACE, DBContract.PlaceTestQuestion.IDTEST, DBContract.PlaceTestQuestion.IDQUESTION,
                DBContract.PlaceTestQuestion.IDPLACE, DBContract.PlaceTestQuestion.IDTEST, DBContract.PlaceTestQuestion.IDQUESTION)

        // Scripts para eliminar las tablas de la base de datos
        private val SQL_DELETE_ENTRIES_SITIO = String.format("drop table if exists %s", DBContract.Place.TABLE_NAME)
        private val SQL_DELETE_ENTRIES_TEST = String.format("drop table if exists %s", DBContract.Test.TABLE_NAME)
        private val SQL_DELETE_ENTRIES_PREGUNTA = String.format("drop table if exists %s", DBContract.Question.TABLE_NAME)
        private val SQL_DELETE_ENTRIES_STP = String.format("drop table if exists %s", DBContract.PlaceTestQuestion.TABLE_NAME)

        private val TAG = "DBHelper"
    }

        private var contexto: Context = context



    override fun onCreate(db: SQLiteDatabase) {
        Log.d(TAG, "onCreate con SQL")

        // Creamos la esctructura de la base de datos
        db.execSQL(SQL_CREATE_ENTRIES_SITIO)
        db.execSQL(SQL_CREATE_ENTRIES_TEST)
        db.execSQL(SQL_CREATE_ENTRIES_PREGUNTA)
        db.execSQL(SQL_CREATE_ENTRIES_STP)


        // Obtenenemos la lista de placeslist a añadir
        var placeslist: ArrayList<Place> = RellenaBD().rellenabdsitios()

        //obtener la lista de tests a añadir
        var listatest: ArrayList<Test> = RellenaBD().rellenabdtest()

        //Añade los sitos a la bd cuando se crea la bd
        rellenabd(placeslist, listatest, db)

    }

    override fun onUpgrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
        // This database is only a cache for online data, so its upgrade policy is
        // to simply to discard the data and start over
        db.execSQL(SQL_DELETE_ENTRIES_SITIO)
        db.execSQL(SQL_DELETE_ENTRIES_TEST)
        db.execSQL(SQL_DELETE_ENTRIES_PREGUNTA)
        db.execSQL(SQL_DELETE_ENTRIES_STP)
        onCreate(db)
    }


    @Throws(SQLiteConstraintException::class)
    fun rellenabd(placeslist: ArrayList<Place>, listatest: ArrayList<Test>, db: SQLiteDatabase): Boolean {

        db.beginTransaction()
        try {
            val values = ContentValues()
            placeslist.forEach { s ->

                // Metemos el sitio que toca
                // Creamos un nuevo mapa de values, where cada nombre de la columna es la clave
                values.put(DBContract.Place.TITLE, s.Name)
                values.put(DBContract.Place.IMAGE, s.Image)
                values.put(DBContract.Place.DESCRIPTION, s.Description)
                values.put(DBContract.Place.LOCATION, s.Location)
                values.put(DBContract.Place.INTERESTTURL, s.InterestUrl)

                // Insertamos la columna, y se devuleve el indice de la columna
                val sitioid = db.insert(DBContract.Place.TABLE_NAME, null, values)
                //Log.d(TAG,String.format("Sitioid:%d \tTitulo:%s Localizacion:%s",sitioid,s.Titulo,s.Localizacion))

                // Metemos el test que toca
                val valuestest = ContentValues()
                val test = listatest[s.PlacesId - 1]
                valuestest.put(DBContract.Test.ID, test.TestId)
                valuestest.put(DBContract.Test.NAME, test.Name)
                valuestest.put(DBContract.Test.SCORE, test.Score)
                db.insert(DBContract.Test.TABLE_NAME, null, valuestest)
                //Log.d(TAG,String.format("Testid:%d \tNombre:%s \tValoracion:%s",test.TestId,test.Nombre,test.Valoracion))

                // Obtenemos todas las preguntas del test y las añadimos a la base de datos
                test.Questions.forEach { p ->
                    val valuesp = ContentValues()
                    val valuesstp = ContentValues()
                    valuesp.put(DBContract.Question.ID, p.Questionid)
                    valuesp.put(DBContract.Question.TEXT, p.Text)
                    valuesp.put(DBContract.Question.ANSWER1, p.Answer1)
                    valuesp.put(DBContract.Question.ANSWER2, p.Answer2)
                    valuesp.put(DBContract.Question.ANSWER3, p.Answer3)
                    valuesp.put(DBContract.Question.ANSWER4, p.Answer4)
                    valuesp.put(DBContract.Question.CORRECTANSWER, p.CorrectAnswer)
                    db.insert(DBContract.Question.TABLE_NAME, null, valuesp)

                    //Log.d(TAG,String.format("PreguntaId:%d \tTexto:%s",p.Preguntaid,p.Texto))

                    // Rellenamos la tabla sitio test pregunta con las id de cada tipo
                    valuesstp.put(DBContract.PlaceTestQuestion.IDPLACE, sitioid)
                    valuesstp.put(DBContract.PlaceTestQuestion.IDTEST, test.TestId)
                    valuesstp.put(DBContract.PlaceTestQuestion.IDQUESTION, p.Questionid)
                    db.insert(DBContract.PlaceTestQuestion.TABLE_NAME, null, valuesstp)
                    Log.d(TAG,String.format("SitioId:%d \tTestID:%d \tPreguntaId:%d",sitioid,test.TestId,p.Questionid))
                }
            }
            db.setTransactionSuccessful()

        } catch (e: SQLiteException) {

            e.printStackTrace()

        } finally {

            db.endTransaction()
            return true
        }
    }





    override fun onDowngrade(db: SQLiteDatabase, oldVersion: Int, newVersion: Int) {
        onUpgrade(db, oldVersion, newVersion)
    }

    /**
     * Función mediante la que obtenmemos un sitio de la base de datos
     * @param placeid Id del sitio que quermos recuperar de la base de datos
     * @return Un array de sitios que tiene el sitio devuelto en la posición 0
     */
    @Throws(SQLiteConstraintException::class)
    fun getPlace(placeid: Int): ArrayList<Place> {
        val sitios = ArrayList<Place>()
        val db = writableDatabase
        var cursor: Cursor?
        try {
            cursor = db.rawQuery("select * from " + DBContract.Place.TABLE_NAME + " WHERE " + DBContract.Place.ID + "=" + placeid, null)
        } catch (e: SQLiteException) {
            e.printStackTrace()
            return ArrayList()
        }

        var Titulo: String
        var Imagen: String
        var Descripcion: String
        var Localizacion: String
        var UrlInteres: String

        if (cursor!!.moveToFirst()) {
            while (cursor.isAfterLast == false) {
                Titulo = cursor.getString(cursor.getColumnIndex(DBContract.Place.TITLE))
                Imagen = cursor.getString(cursor.getColumnIndex(DBContract.Place.IMAGE))
                Descripcion = cursor.getString(cursor.getColumnIndex(DBContract.Place.DESCRIPTION))
                Localizacion = cursor.getString(cursor.getColumnIndex(DBContract.Place.LOCATION))
                UrlInteres = cursor.getString(cursor.getColumnIndex(DBContract.Place.INTERESTTURL))

                sitios.add(Place(PlacesId = placeid, Name = Titulo, Image = Imagen, Description = Descripcion, Location = Localizacion, InterestUrl = UrlInteres))
                cursor.moveToNext()
            }
        }
        return sitios
    }

    /**
     * Función que devuelve un test con su lista de preguntas
     * @param sitioid Id del sitio del que se devuelve una lista de test
     * @return Se devuelve un ArrayList de test que contiene tambien la lista de preguntas
     */
    @Throws(SQLiteConstraintException::class)
    fun getTest(sitioid: Int): ArrayList<Test> {
        val test = ArrayList<Test>()
        val db = writableDatabase
        var cursor: Cursor?
        try {

            // Query sql
            /*select t.Testid,t.Nombre,t.Valoracion,p.Preguntaid,p.Texto,p.Respuesta1,p.Respuesta2,p.Respuesta3,p.Respuesta4,p.Solucion
            from SITIO_TEST_PREGUNTA stp
            inner join PREGUNTA p on stp.IdPregunta = p.Preguntaid
            inner join TEST t on stp.IdTest = t.Testid
            where stp.IdSitio=2*/

            // Obtenemos las preguntas del test asosciado al sitio
            val query: String = String.format("select t.%s,t.%s,t.%s," +
                    "p.%s,p.%s,p.%s,p.%s,p.%s,p.%s,p.%s " +
                    "from %s stp " +
                    "inner join %s p on stp.%s = p.%s " +
                    "inner join %s t on stp.%s = t.%s " +
                    "where stp.%s = %s",
                    DBContract.Test.ID, DBContract.Test.NAME, DBContract.Test.SCORE,
                    DBContract.Question.ID, DBContract.Question.TEXT, DBContract.Question.ANSWER1, DBContract.Question.ANSWER2, DBContract.Question.ANSWER3, DBContract.Question.ANSWER4, DBContract.Question.CORRECTANSWER,
                    DBContract.PlaceTestQuestion.TABLE_NAME,
                    DBContract.Question.TABLE_NAME, DBContract.PlaceTestQuestion.IDQUESTION, DBContract.Question.ID,
                    DBContract.Test.TABLE_NAME, DBContract.PlaceTestQuestion.IDTEST, DBContract.Test.ID,
                    DBContract.PlaceTestQuestion.IDPLACE, sitioid)
            //Log.d(TAG,query)
            cursor = db.rawQuery(query, null)

        } catch (e: SQLiteException) {
            e.printStackTrace()
            return ArrayList()
        }

        var testId: Int
        var nombre: String
        var valoracion: String
        var preguntas = ArrayList<Question>()
        var preguntaid: Int
        var texto: String
        var respuesta1: String
        var respuesta2: String
        var respuesta3: String
        var respuesta4: String
        var solucion: String

        // Añadimos las preguntas al Array
        if (cursor!!.moveToFirst()) {
            testId = cursor.getInt(cursor.getColumnIndex(DBContract.Test.ID))
            nombre = cursor.getString(cursor.getColumnIndex(DBContract.Test.NAME))
            valoracion = cursor.getString(cursor.getColumnIndex(DBContract.Test.SCORE))
            while (cursor.isAfterLast == false) {
                preguntaid = cursor.getInt(cursor.getColumnIndex(DBContract.Question.ID))
                texto = cursor.getString(cursor.getColumnIndex(DBContract.Question.TEXT))
                respuesta1 = cursor.getString(cursor.getColumnIndex(DBContract.Question.ANSWER1))
                respuesta2 = cursor.getString(cursor.getColumnIndex(DBContract.Question.ANSWER2))
                respuesta3 = cursor.getString(cursor.getColumnIndex(DBContract.Question.ANSWER3))
                respuesta4 = cursor.getString(cursor.getColumnIndex(DBContract.Question.ANSWER4))
                solucion = cursor.getString(cursor.getColumnIndex(DBContract.Question.CORRECTANSWER))

                preguntas.add(Question(Questionid = preguntaid, Text = texto, Answer1 = respuesta1,
                        Answer2 = respuesta2, Answer3 = respuesta3, Answer4 = respuesta4,
                        CorrectAnswer = solucion))
                //Log.d(TAG,preguntas.toString())
                cursor.moveToNext()
            }
            //Rellenamos el array que contiene el test
            test.add(Test(TestId = testId, Name = nombre, Score = valoracion, Questions = preguntas))
            //Log.d(TAG,test.toString())
        }
        return test
    }

    /**
     * Función que devuelve un ArrayList con todos los sitios
     * @return ArrayList de con todos los sitos en la BD
     */
    @Throws(SQLiteConstraintException::class)
    fun readAllSitios(): ArrayList<Place> {

        val sitios = ArrayList<Place>() //Variable de retorno
        val db = writableDatabase
        var cursor: Cursor?
        try {
            cursor = db.rawQuery("select * from " + DBContract.Place.TABLE_NAME, null)
        } catch (e: SQLiteException) {
            e.printStackTrace()
            return ArrayList()
        }

        var sitioid: Int
        var Titulo: String
        var Imagen: String
        var Descripcion: String
        var Localizacion: String
        var UrlInteres: String

        // Accedemos a cada columna
        if (cursor!!.moveToFirst()) {
            while (cursor.isAfterLast == false) {

                sitioid = cursor.getInt(cursor.getColumnIndex(DBContract.Place.ID))
                Titulo = cursor.getString(cursor.getColumnIndex(DBContract.Place.TITLE))
                Imagen = cursor.getString(cursor.getColumnIndex(DBContract.Place.IMAGE))
                Descripcion = cursor.getString(cursor.getColumnIndex(DBContract.Place.DESCRIPTION))
                Localizacion = cursor.getString(cursor.getColumnIndex(DBContract.Place.LOCATION))
                UrlInteres = cursor.getString(cursor.getColumnIndex(DBContract.Place.INTERESTTURL))

                // Añadimos el sitio de la base de datos al ArrayList
                sitios.add(Place(PlacesId = sitioid, Name = Titulo, Image = Imagen,
                        Description = Descripcion, Location = Localizacion,
                        InterestUrl = UrlInteres))
                cursor.moveToNext()
            }
        }
        return sitios
    }

    /**
     * Actualiza la valoración del test donde la id es la dada
     * @param testId Test que hay que modifica
     * @param valoracion del Test que hay que poner
     */
    fun updateTestVal(testId: Int, valoracion: String): Boolean {

        val db = writableDatabase
        try {
            var sql = String.format("update %s set %s=%s where %s=%d",
                    DBContract.Test.TABLE_NAME,
                    DBContract.Test.SCORE, valoracion,
                    DBContract.Test.ID, testId)
            db.execSQL(sql)
        } catch (e: SQLiteException) {
            e.printStackTrace()
            return false
        }
        return true
    }


    /**
     * Función que devuelve una lista de todos los test de la BD sin las preguntas
     */
    fun getAllTest(): ArrayList<Test> {

        val testlist = ArrayList<Test>()
        val db = writableDatabase
        var cursor: Cursor?
        try {
            cursor = db.rawQuery("select * from " + DBContract.Test.TABLE_NAME, null)
        } catch (e: SQLiteException) {
            e.printStackTrace()
            return ArrayList()
        }

        var testid: Int
        var testname: String
        var testscore: String

        if (cursor!!.moveToFirst()) {
            while (cursor.isAfterLast == false) {

                testid = cursor.getInt(cursor.getColumnIndex(DBContract.Test.ID))
                testname = cursor.getString(cursor.getColumnIndex(DBContract.Test.NAME))
                testscore = cursor.getString(cursor.getColumnIndex(DBContract.Test.SCORE))

                testlist.add(Test(testid, testname, testscore, ArrayList()))
                cursor.moveToNext()
            }
        }
        return testlist
    }
}


