package guillermocs.patrimonio30

import android.os.Bundle
import android.speech.tts.TextToSpeech
import android.speech.tts.TextToSpeech.OnInitListener
import android.speech.tts.TextToSpeech.getMaxSpeechInputLength
import android.support.design.widget.FloatingActionButton
import android.support.v4.app.Fragment
import android.text.TextUtils
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.bumptech.glide.Glide

import java.util.*

/**
 * @author Guillermo Cebrián Serrano
 * Clase que que nos muestra la informacion sobre un sitio
 * se usa el metodo newInstance para crear una nueva instancia
 */

class InfoFragment : Fragment(){



    private lateinit var tvTitle: TextView // Vista que almacena el titulo del sitio
    private lateinit var tvDescription: TextView  // Vista que almacena la descrición
    private lateinit var tvImage: ImageView // Vista que almacena la imagen

    private lateinit var fabTest: FloatingActionButton // Boton que nos lleva a TestFragment
    private lateinit var fabTts: FloatingActionButton // Boton que inicializa la lectura de la descripcion con TTS
    private lateinit var fabInfo: FloatingActionButton // Boton que nos lleva a TravelFragment

    private var placeid: Int = 0 // Id del sitio que se va a mostrar
    private lateinit var textToSpeech: TextToSpeech // Variable para leer el texto


    companion object {

        // Creamos las variables
        // Para mostrar en la salida de los logs
        private var TAG = "InfoFragment"
        private const val ARG_PLACEID = "SitioId"

        /**
         * @param PlaceId Sitio del que vamos a mostrar la informacion
         * @return Una nueva instancia de InfoFragment
         */

        @JvmStatic
        fun newInstance(PlaceId: Int) =
                InfoFragment().apply {
                    arguments = Bundle().apply {
                        putInt(ARG_PLACEID, PlaceId)
                    }
                }
    }

    /**
     * Metodo que recibe la id del sitio y lo asigna al atributo placeid
     * @param savedInstanceState Recibe un bundle con los datos que se le pasan a InfoFragment
     */
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            placeid = it.getInt(ARG_PLACEID)
        }
    }

    /**
     *
     */

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val v = inflater.inflate(R.layout.fragment_info, container, false)

        // Enlazar los XML con sus variables
        tvTitle = v.findViewById(R.id.tvTitulo) as TextView
        tvDescription = v.findViewById(R.id.tvDescripcion) as TextView
        tvImage = v.findViewById(R.id.ivFotografía) as ImageView

        fabTest = v.findViewById(R.id.fabTest) as FloatingActionButton
        fabTts = v.findViewById(R.id.fabTts) as FloatingActionButton
        fabInfo = v.findViewById(R.id.fabInfo) as FloatingActionButton

        // Obtenemos los datos a mostrar
        val auxplace = MainActivity.getDbHelper().getPlace(placeid)[0]
        // Log.d(TAG,String.format("Auxsitio de Titulo: %s \t Localizacion %s",auxplace.Titulo,auxplace.Localizacion))

        // Rellenamos la vista con los datos
        tvTitle.text = auxplace.Name
        tvDescription.text = auxplace.Description
        tvImage.loadUrl(auxplace.Image)
        Log.d(TAG, "Place cargado")




        // Incializamos el TTS
        textToSpeech = TextToSpeech(context, OnInitListener {
            fun onInit(status: Int) {
                var language = Locale("spa", "ESP")
                if (status != TextToSpeech.ERROR) textToSpeech.setLanguage(language)
            }
        })

        // Cuando pulsamos el boton test, iniciamos un nuevo TestFragment con la id del sitio
        fabTest.setOnClickListener {
            // Pasamos al fragment del test
            fragmentManager!!.beginTransaction().replace(R.id.fragment_container, TestFragment.newInstance(placeid)).addToBackStack(TAG).commit()
        }

        // Cuando pulsamos el boton TTS, leemos ladescripcion por la salida de audio
        fabTts.setOnClickListener {

            if (textToSpeech != null) textToSpeech.stop()

            val utteranceId = this.hashCode().toString() + ""

            // Divide el texto en partes si supera el maximo numero de caracteres de la funcion TTS
            val length = TextToSpeech.getMaxSpeechInputLength() -1
            var text =  tvDescription.text.chunked(length)
            text.forEach({
                //Log.d(TAG,it)
            textToSpeech.speak(it, TextToSpeech.QUEUE_ADD, null, utteranceId)
        })

        }

        // Cuando pulsamos el boton Info, iniciamos un fragment que nos muestra como llegar al sitio
        // con la ubicación que le pasamos y nos muestra urls de interes
        fabInfo.setOnClickListener {
            fragmentManager!!.beginTransaction().replace(R.id.fragment_container,
                    TravelFragment.newInstance(auxplace.Location, auxplace.InterestUrl)).addToBackStack(TAG).commit()
        }




        return v
    }

    /**
     * Cuando se detiene el fragment se detiene el TTS
     */
    override fun onPause() {
        if (textToSpeech != null) {
            textToSpeech.stop()
            textToSpeech.shutdown()
        }
        super.onPause()
    }


    /**
     * Cargamos la imagen del sitio dentro del ImageView
     */
    private fun ImageView.loadUrl(url: String) {
        Log.d(TAG, String.format("Loadurl: %s", url))
        Glide.with(context).load(url).into(this)
    }





}

