package guillermocs.patrimonio30

import android.content.Context
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.google.android.gms.maps.SupportMapFragment
import guillermocs.patrimonio30.Clases.Test


/**
 * @author Guillermo Cebrián Serrano
 * Fragment que se encarga de mostrar una lista de todos los test
 */
class ListTestFragment : Fragment() {

    lateinit var mRecyclerView: RecyclerView // Contenedor de la lista de Test
    val mAdapterTest = RecyclerAdapterTest() // Adaptador que se encarga de colocar los datos dentro del RecyclerView

    companion object {
        const val TAG = "ListTestFragment"
        lateinit var fm: FragmentManager
    }


    /**
     * Metodo que se encarga de generar la vista del fragment
     * @ return v que es la vista ya creada
     */
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {

        // Rellenamos el layout para este Fragment
        var v = inflater.inflate(R.layout.fragment_testlist, container, false)

        // Inicializamos el fragmentManager
        fm = requireFragmentManager()

        // Preparamos el RecyclerView con la vista que hemos creado
        setUpRecyclerView(v)
        return v
    }

    /**
     * Funcion que se encarga de rellenar la vista del fragment con una lista de test
     * @param view Vista creada por el fragment
     */
    private fun setUpRecyclerView(view: View) {

        // Enlazamos con el xml
        mRecyclerView = view.findViewById(R.id.rvAchievemenstsList) as RecyclerView
        mRecyclerView.setHasFixedSize(true)
        mRecyclerView.layoutManager = LinearLayoutManager(context)

        // Obtenemos la lista de test de la BD
        val testlist: ArrayList<Test> = MainActivity.getDbHelper().getAllTest()

        // Inicializamos el adaptador de los test con la lista de test, y lo asignamos al RecyclerView
        mAdapterTest.RecyclerAdapterSitios(testlist, requireContext())
        mRecyclerView.adapter = mAdapterTest
    }


    /**
     * Clase que se encarga de adaptar una lista de test a un RecyclerView
     */
    class RecyclerAdapterTest : RecyclerView.Adapter<RecyclerAdapterTest.ViewHolder>() {

        var test: ArrayList<Test> = ArrayList() // Lista de Test
        lateinit var context: Context // Contexto

        /**
         * Metodo de inicializacion
         */
        fun RecyclerAdapterSitios(test: ArrayList<Test>, context: Context) {
            this.test = test
            this.context = context
        }

        /**
         * Metodo que enlaza cada test con el ViewHolder
         */
        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val item = test.get(position)
            holder.bind(item, context)
        }

        /**
         * Creamos el ViewHolder con su xml
         */
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            val layoutInflater = LayoutInflater.from(parent.context)
            return ViewHolder(layoutInflater.inflate(R.layout.fragment_testlist_list, parent, false))
        }

        /**
         * @return El tamaño de la lista de test
         */
        override fun getItemCount(): Int {
            return test.size
        }

        /**
         * Clase que enlaza cada test con sus vistas
         */

        class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
            private val ivTestImage = view.findViewById(R.id.list_test_image) as ImageView
            private val tvTestName = view.findViewById(R.id.list_test_text_name) as TextView
            private val tvTestScore = view.findViewById(R.id.list_test_text_score) as TextView


            /**
             * Metdodo que enlaza cada test con sus vistas
             */
            fun bind(test: Test, context: Context) {

                var score = test.Score.toDouble() * 10 // Valoracion del test sobre 10 en formato double

                // Asignamos los valores del test a sus vistas
                tvTestName.text = test.Name
                tvTestScore.text = score.toString() + "/10"

                // Rellenamos la imagen del test en funcion de la puntuacion
                if (score < 5) ivTestImage.setImageResource(R.drawable.ic_wood_medal)
                if (score >= 5 && score < 7) ivTestImage.setImageResource(R.drawable.ic_bronze_medal)
                if (score >= 7 && score < 8.5) ivTestImage.setImageResource(R.drawable.ic_silver_medal)
                if (score >= 8.5) ivTestImage.setImageResource(R.drawable.ic_gold_medal)

                // Cunado se selecciona un test vamos al test
                itemView.setOnClickListener({
                    fm.beginTransaction().replace(R.id.fragment_container,
                            TestFragment.newInstance(test.TestId))
                            .addToBackStack(ListTestFragment.TAG).commit()
                })
            }

        }

    }
}



