package guillermocs.patrimonio30

import android.content.Intent
import android.graphics.Bitmap
import android.net.Uri
import android.os.AsyncTask
import android.os.Bundle
import android.support.constraint.ConstraintLayout
import android.support.design.widget.NavigationView
import android.support.v4.view.GravityCompat
import android.support.v7.app.ActionBarDrawerToggle
import android.support.v7.app.AppCompatActivity
import android.text.TextUtils
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import com.bumptech.glide.Glide
import com.bumptech.glide.request.RequestOptions
import com.google.android.gms.maps.CameraUpdateFactory
import com.google.android.gms.maps.GoogleMap
import com.google.android.gms.maps.GoogleMap.OnMarkerClickListener
import com.google.android.gms.maps.OnMapReadyCallback
import com.google.android.gms.maps.SupportMapFragment
import com.google.android.gms.maps.model.BitmapDescriptorFactory
import com.google.android.gms.maps.model.LatLng
import com.google.android.gms.maps.model.Marker
import com.google.android.gms.maps.model.MarkerOptions
import com.twitter.sdk.android.core.*
import com.twitter.sdk.android.core.models.User
import com.twitter.sdk.android.tweetcomposer.ComposerActivity
import guillermocs.patrimonio30.Clases.Place
import kotlinx.android.synthetic.main.activity_main.*

import kotlinx.android.synthetic.main.app_bar_main.*
import java.io.BufferedReader
import java.io.IOException
import java.io.InputStream
import java.io.InputStreamReader

/**
 * @author Guillermo Cebrián Serrano
 * Clase principal de la aplicación desde aqui se inicializa un mapa y se crean las llamadas a las
 * demas clases
 */

class MainActivity : AppCompatActivity(), NavigationView.OnNavigationItemSelectedListener,
        OnMapReadyCallback, OnMarkerClickListener {



    companion object {
        const val TAG = "MActivity" // Tag para los logs
        val VALLADOLID = LatLng(41.652251, -4.7245) // Constante para centrar el mapa

        private lateinit var DBHelper: DBHelper // DBHelper con el que accedemos a la base de datos

        /**
         * Metodos con el que se obtiene el DBHelper
         * @return DBHelper de la clase MainActivity
         */
        fun getDbHelper(): DBHelper {
            return DBHelper
        }

    }

    private lateinit var map: GoogleMap // Variable que representa el mapa dentro del código

    // Lista donde almacenamos marcadores y la id del sitio al que corresponden
    private val markers = HashMap<Marker, Int>()

    // Variable que invoca al supportFrgamentManager
    private var fm = supportFragmentManager

    private lateinit var navigationView: NavigationView // Variable que representa el NavigationView
    private lateinit var nav_Menu: Menu // Variable que representa el menu del NView

    private lateinit var nav_HeaderMenu: View //Variable que representa la cabecera del NView
    private lateinit var tvName_Header: TextView // Variable de la cabecera TextView Nombre
    private lateinit var tvEmail_Header: TextView // Variable de la cabecera TextView Email
    private lateinit var ivUserImage_Header: ImageView // Variable de la cabezera ImageView Imagen

    private lateinit var mapFragment: SupportMapFragment //Fragment que soporta el mapa

    /**
     * Metodo que se llama cuando se inicia la aplicación.
     */
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        // Inicializamos Twitter
        val config = TwitterConfig.Builder(applicationContext)
                .logger(DefaultLogger(Log.DEBUG))
                .twitterAuthConfig(TwitterAuthConfig(getString(R.string.OAuthConsumerKey), getString(R.string.OAuthConsumerSecret)))
                .debug(true)
                .build()
        Twitter.initialize(config)

        // Incicializamos la vista
        setContentView(R.layout.activity_main)

        // Enlazamos el navigation drawer a las variables del código
        navigationView = findViewById(R.id.nav_view)
        nav_Menu = navigationView.menu
        nav_HeaderMenu = navigationView.getHeaderView(0)

        tvName_Header = nav_HeaderMenu.findViewById(R.id.nav_TextViewUser) as TextView
        tvEmail_Header = nav_HeaderMenu.findViewById(R.id.nav_TextViewEmail) as TextView
        ivUserImage_Header = nav_HeaderMenu.findViewById(R.id.nav_ImageView) as ImageView

        // Obtenemos la sesion de Twitter para configurar la vista
        var session: TwitterSession? = TwitterCore.getInstance().sessionManager.activeSession
        UiSession(session)

        // Se inicializa el NavigationDrawer y se pone a la espera de que se pulse alguna opción
        setSupportActionBar(toolbar)
        val toggle = ActionBarDrawerToggle(
                this, drawer_layout, toolbar, R.string.navigation_drawer_open, R.string.navigation_drawer_close)
        drawer_layout.addDrawerListener(toggle)
        toggle.syncState()

        nav_view.setNavigationItemSelectedListener(this)

        //Obtiene el fragmento que soporta el mapa y es notificado cuando el mapa esta listo para ser usado
        mapFragment = supportFragmentManager
                .findFragmentById(R.id.map) as SupportMapFragment

        mapFragment.getMapAsync(this)

        // Inicializamos el DBHelper
        DBHelper = DBHelper(this.applicationContext)
        Log.d(TAG,"Antes de Twitter get session")
    }







    /**
     * Vuelve al fragment anterior cuando se  pulsa el boton atras
     */
    override fun onBackPressed() {

        val count = fm.backStackEntryCount
        centerMap()
        Log.d(TAG,"Count " + count.toString())
        if (count == 0) {
            super.onBackPressed()
            fm.popBackStack()

        } else {
            if(count==1) fm.beginTransaction().show(mapFragment).commit()
            fm.popBackStack()
        }

    }

    /**
     * Metodo que rellena el menu añadiendo opciones al ActionBar si esta visible
     * @return true si se puede rellenar el menu
     */
    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        //menuInflater.inflate(R.menu.main, menu)
        return true
    }

    /**
     * Maneja los clicks en la ActionBar. En nuestro caso el click en el botton de opciones
     */
    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.action_settings -> {
                startActivity(Intent(this, SettingsActivity::class.java))
                return true
            }
            else -> return super.onOptionsItemSelected(item)
        }
    }


    /**
     * Maneja los los del NavigationDrawer y llama a los distintos fragments
     */
    override fun onNavigationItemSelected(item: MenuItem): Boolean {

        when (item.itemId) {
            R.id.nav_places -> {
                hideMap()
                // Inicia un nuevo fragment que muestra la lista de placeslist
                fm.beginTransaction().replace(R.id.fragment_container, PlacesFragment()).addToBackStack(TAG).commit()
            }
            R.id.nav_achievements -> {
                hideMap()
                // Inicia un nuevo fragment que muestra la lista de logros
                fm.beginTransaction().replace(R.id.fragment_container, ListTestFragment()).addToBackStack(TAG).commit()

            }
            R.id.nav_news -> {
                // Inicia una nueva activity que muestra una lista de noticias
                startActivity(Intent(this,NewsActivity().javaClass))

            }
            R.id.nav_share -> {
                var session: TwitterSession = TwitterCore.getInstance().sessionManager.activeSession
                val intent = ComposerActivity.Builder(this)
                        .session(session)
                        .text("")
                        .hashtags("#patrimoniocyl")
                        .createIntent()
                startActivity(intent)
            }
            R.id.nav_login -> {
                startActivity(Intent(this,LoginTwitter().javaClass))
            }
        }

        drawer_layout.closeDrawer(GravityCompat.START)
        return true
    }

    private fun hideMap() {
        fm.beginTransaction().hide(mapFragment).commit()
    }

    /**
     * Metodo que se llama cuando el mapa ya esta listo para usarse.
     * @param googleMap Variable que contiene el mapa ya listo para usarse
     */
    override fun onMapReady(googleMap: GoogleMap) {
        // Asignamos el valor del mapa creado a nuestra variable map
        map = googleMap


        // map.mapType = GoogleMap.MAP_TYPE_HYBRID // Cambiamos el tipo de mapa, para que se vea mas claro
        map.uiSettings.isMapToolbarEnabled = false // Quitamos la barra de herramientas del mapa

        // Centramos el mapa
        centerMap()

        // Obtenemos la lista de placeslist de la base de datos
        var placesofbd = DBHelper.readAllSitios()
        Log.d(TAG, "Place de la base de datos obtenidos")

        // Llenamos el mapa con todos los placeslist
        placesofbd.forEach {
            // Log.d(TAG, String.format("Place antes de async Id: %d \tTitulo: %s \t Localizacion %s", it.Sitioid, it.Titulo, it.Localizacion))
            fillMap(it).execute()
            // Log.d(TAG, "Place Cargado en el mapa async")
        }

        // Preparamos los marcadores para recibir clicks
        map.setOnMarkerClickListener(this)

    }

    /**
     * Funcion que se llama cuando se pulsa un marcador
     * @param marker Marcador pulsado
     */
    override fun onMarkerClick(marker: Marker?): Boolean {
        Log.d(TAG, "OnMarkerClick")

        // Obtenemos el sitio asociado al marcador
        var sitioid = markers[marker]!!

        hideMap()
        // Iniciamos una nueva instancia de InfoFragment para mostrar la información del sitio asociado
        fm.beginTransaction().replace(R.id.fragment_container, InfoFragment.newInstance(sitioid)).addToBackStack(TAG).commit()
        return false
    }


    /**
     * Funcion que centra el mapa en VALLADOLID
     */
    private fun centerMap() {
        map.moveCamera(CameraUpdateFactory.newLatLngZoom(VALLADOLID, 7.5f))
    }



    override fun onResume() {
        super.onResume()
        var session: TwitterSession? = TwitterCore.getInstance().sessionManager.activeSession
        UiSession(session)

    }

    override fun onPause() {
        super.onPause()
    }


    private fun UiSession(session: TwitterSession?) {
        Log.d(TAG, "session = " + session.toString())
        if (session == null) {

            tvName_Header.visibility = View.INVISIBLE
            tvEmail_Header.visibility = View.INVISIBLE
            ivUserImage_Header.visibility = View.INVISIBLE

            nav_Menu.findItem(R.id.nav_login).setTitle(R.string.nav_login)
            nav_Menu.findItem(R.id.nav_share).isVisible = false
            Toast.makeText(applicationContext, "No logueado", Toast.LENGTH_LONG)
            Log.d(TAG, "El usuario no esta logueado")

        } else {

            nav_Menu.findItem(R.id.nav_login).setTitle(R.string.nav_logout)
            nav_Menu.findItem(R.id.nav_share).isVisible = true
            //TODO: Llamarlo solo cuando se necesite
            getUserDetailsFromTwitter()
        }
    }



    private fun getUserDetailsFromTwitter() {
        var user = TwitterCore.getInstance()
                .apiClient
                .accountService
                .verifyCredentials(true, false,false)
        user.enqueue(object : Callback<User>() {
            override fun success(userResult: Result<User>) {

                tvName_Header.text = userResult.data.name
                tvEmail_Header.text = userResult.data.email
                Glide.with(applicationContext)
                        .load(userResult.data.profileImageUrl.replace("_normal", ""))
                        .apply(RequestOptions.circleCropTransform())
                        .into(ivUserImage_Header)

                tvName_Header.visibility = View.VISIBLE
                tvEmail_Header.visibility = View.VISIBLE
                ivUserImage_Header.visibility = View.VISIBLE

                Log.d(TAG, "User "+user.toString())
            }

            override fun failure(e: TwitterException) {
                Log.d("TwitterKit", "Verify Credentials Failure", e);
            }
        })
    }



    /**
     * Clase interna de tipo AsyncTask que se encarga de rellenar el mapa con los placeslist
     * @param asyncsitio Place que recibimos para insertarlo en el mapa
     */
    inner class fillMap(asyncsitio: Place) : AsyncTask<Void, Void, Bitmap>() {

        var myplace = asyncsitio // Atributo que guarda el sitio

        /**
         * Metodo que se realiza en segundo plano. Obtenemos la url de la imagen del sitio y obtenemos
         * un recurso bmp
         * @return bmp Devuelve la imagen del sito en formato bmp
         */
        override fun doInBackground(vararg params: Void): Bitmap? {

            var bmp: Bitmap? = null // Variable donde se almacena la imagen

            try {
                val uri = Uri.parse(myplace.Image) // Urls de la imagen

                // Obtenemos la imagen desde la Url con la librería Glide y reducimos su tamaño
                bmp = Glide.with(applicationContext)
                        .asBitmap()
                        .load(uri)
                        .submit(70, 70)
                        .get()
            } catch (e: Exception) {
                e.printStackTrace()
            }

            return bmp

        }

        /**
         * Metodo que se ejecuta cuando la imagen ya esta descargada en formato bmp.
         * Configuramos el marcador ,lo añadimos al mapa y lo emparejamos con el sitio.
         * @param result Imagen descargada en formato bmp
         */
        override fun onPostExecute(result: Bitmap?) {
            super.onPostExecute(result)
            val Titulo = myplace.Name // Nombre del sitio
            val Ubicacion = myplace.Location // Localizacion del sitio en formato String

            // Separamos el String que nos da la ubciacion en las coordenadas
            val latlong = Ubicacion.split(",".toRegex()).dropLastWhile { it.isEmpty() }.toTypedArray()
            val latitude = java.lang.Double.parseDouble(latlong[0])
            val longitude = java.lang.Double.parseDouble(latlong[1])

            // Creamos una instancia de MarkersOption
            var markerOptions = MarkerOptions()

            // Configuramos el MarkerOptions con Titulo, Ubicación y Imagen del sitio
            if(result == null) {
                markerOptions.position(LatLng(latitude, longitude))
                        .title(Titulo)
            }else{
                markerOptions.position(LatLng(latitude, longitude))
                        .title(Titulo)
                        .icon(BitmapDescriptorFactory.fromBitmap(result))
            }

            // Añadimos el sitio al mapa y a la lista de marcadores
            var marker = map.addMarker(markerOptions)
            markers[marker] = myplace.PlacesId

        }
    }

}



