package guillermocs.patrimonio30


import android.content.Context
import android.os.Bundle
import android.support.v4.app.Fragment
import android.support.v4.app.FragmentManager
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import com.bumptech.glide.Glide
import guillermocs.patrimonio30.Clases.Place


/**
 * @author Guillermo Cebrián Serrano
 * Fragment que se encarga de mostrar una lista de todos los sitios
 */
class PlacesFragment : Fragment() {

    lateinit var mRecyclerView: RecyclerView // Contenedor de la lista de sitios
    val mAdapterPlaces = RecyclerAdapterPlaces() // Adaptador que se encarga de colocar los datos dentro del RecyclerView


    companion object {
        const val TAG = "PlacesFragment"
        lateinit var fm: FragmentManager
    }

    /**
     * Metodo que se encarga de generar la vista del fragment
     * @ return v que es la vista ya creada
     */
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {

        // Rellenamos el layout para este Fragment
        var v = inflater.inflate(R.layout.fragment_places, container, false)

        // Inicializamos el fragmentManager
        fm = requireFragmentManager()

        // Preparamos el RecyclerView con la vista que hemos creado
        setUpRecyclerView(v)
        return v
    }

    /**
     * Funcion que se encarga de rellenar la vista del fragment con una lista de sitios
     * @param view Vista creada por el fragment
     */
    private fun setUpRecyclerView(view: View) {

        // Enlazamos con el xml
        mRecyclerView = view.findViewById(R.id.rvPlacesList) as RecyclerView
        mRecyclerView.setHasFixedSize(true)
        mRecyclerView.layoutManager = LinearLayoutManager(context)

        // Obtenemos la lista de sitios de la BD
        val placeslist = MainActivity.getDbHelper().readAllSitios()

        // Inicializamos el adaptador de los test con la lista de sistos, y lo asignamos al RecyclerView
        mAdapterPlaces.RecyclerAdapterPlaces(placeslist, this.requireContext())
        mRecyclerView.adapter = mAdapterPlaces
    }

    /**
     * Clase que se encarga de adaptar una lista de sitios a un RecyclerView
     */
    class RecyclerAdapterPlaces : RecyclerView.Adapter<RecyclerAdapterPlaces.ViewHolder>() {

        var placeslist: ArrayList<Place> = ArrayList() // Lista de sitios
        lateinit var context: Context

        /**
         * Metodo de inicializacion
         */
        fun RecyclerAdapterPlaces(placeslist: ArrayList<Place>, context: Context) {
            this.placeslist = placeslist
            this.context = context
        }

        /**
         * Metodo que enlaza cada sitio con el ViewHolder
         */
        override fun onBindViewHolder(holder: ViewHolder, position: Int) {
            val item = placeslist.get(position)
            holder.bind(item, context)
        }

        /**
         * Creamos el ViewHolder con su xml
         */
        override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
            val layoutInflater = LayoutInflater.from(parent.context)
            return ViewHolder(layoutInflater.inflate(R.layout.fragment_places_list, parent, false))
        }

        /**
         * @return El tamaño de la lista de test
         */
        override fun getItemCount(): Int {
            return placeslist.size
        }


        //TODO: Usar solo un list_item y list con PlacesList y Logros list
        /**
         * Clase que enlaza cada test con sus vistas
         */
        class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
            val ivPlaceImage = view.findViewById(R.id.list_places_image) as ImageView
            val tvPlaceName = view.findViewById(R.id.list_place_text_name) as TextView
            val tvPlaceLocation = view.findViewById(R.id.list_place_text_location) as TextView


            /**
             * Metdodo que enlaza cada test con sus vistas
             */
            fun bind(place: Place, context: Context) {

                // Asignamos los valores del sitio a sus vistas
                tvPlaceName.text = place.Name
                tvPlaceLocation.text = place.Location
                ivPlaceImage.loadUrl(place.Image)

                // Cunado se selecciona un sitio vamos obtenemos la informacion de ese sitio
                itemView.setOnClickListener({
                    fm.beginTransaction().replace(R.id.fragment_container,
                            InfoFragment.newInstance(place.PlacesId))
                            .addToBackStack(PlacesFragment.TAG).commit()
                })
            }

            /**
             * Metodo que convierte una url en una imagen, y la coloca en su vista
             * @param url Url de la imagen
             */
            fun ImageView.loadUrl(url: String) {
                Glide.with(context).load(url).into(this)
            }
        }

    }
}



