package guillermocs.patrimonio30

import android.os.Bundle
import android.support.v4.app.Fragment
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.*
import guillermocs.patrimonio30.Clases.Question
import guillermocs.patrimonio30.Clases.Test


/**
 * @author Guillermo Cebrián Serrano
 * Fragment que se encarga de mostrar el Test de un sitio al usuario
 * se usa el metodo newInstance para crear una nueva instancia
 */

class TestFragment : Fragment() {

    companion object {

        const val TEST_FRAGMENT_PLACE_ID = "PlaceId"
        const val TAG = "TestFragment"

        /**
         * Metodo que crea una nueva instancia de tipo TestFragment
         * @param placeid Id del sitio del que queremos obtener el Test
         */
        @JvmStatic
        fun newInstance(placeid: Int) =
                TestFragment().apply {
                    arguments = Bundle().apply {
                        putInt(TEST_FRAGMENT_PLACE_ID, placeid)
                    }
                }
    }

    private var placeid: Int = 0 // Sitio al que se refiere el test

    private val ids_answers: IntArray = intArrayOf(R.id.answer1, R.id.answer2, R.id.answer3,
            R.id.answer4)  // Posibles respuestas del test

    private var correct_answer: Int = 0 //Respuesta correcta
    private var current_question: Int = 0 // Pregunta actual

    private lateinit var answer_is_correct: ArrayList<Boolean> // Array que almacena para cada pregunta respondida, si se ha respondido bien o mal
    private lateinit var test: ArrayList<Test> // Variable que almacena el test
    private lateinit var all_questions: ArrayList<Question> // Array que almacena todas las preguntas de un test
    private lateinit var answers: ArrayList<String> //Array que almacena las respuestas y solucion de una pregunta
    private lateinit var user_answer: ArrayList<Int> // Array que almacena las respuestas que va marcando el usuario

    // Variables de la vista
    private lateinit var tv_test_question: TextView
    private lateinit var rg_group: RadioGroup
    private lateinit var btn_next: Button
    private lateinit var btn_prev: Button
    private lateinit var tv_test_name: TextView
    private lateinit var tableResult : TableLayout

    /**
     * Al crearse el fragment obtenemos la id del sitio
     */
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            placeid = it.getInt(TEST_FRAGMENT_PLACE_ID)
        }

        // Inicializamos variables
        current_question = 0
        answer_is_correct = ArrayList()
        user_answer = ArrayList()

        // Conseguimos el test especifico que vamos a mostrar desde la bd
        test = MainActivity.getDbHelper().getTest(placeid)

        // Obtenemos la lista de preguntas del test
        all_questions = test[0].Questions

        // Iniciamos las respuestas y resultado del usuario a -1 y false
        for (i in all_questions.indices) {
            user_answer.add(i, -1)
            answer_is_correct.add(i,false)
        }

    }

    /**
     * Metodo que crea la vista
     */
    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        val view = inflater.inflate(R.layout.fragment_test, container, false)

        // Inicializamos la vista y asignamos los widgets a las variables
        tv_test_question = view.findViewById(R.id.test_question)
        rg_group = view.findViewById(R.id.answer_group)
        btn_next = view.findViewById(R.id.btn_check)
        btn_prev = view.findViewById(R.id.btn_prev)
        tv_test_name = view.findViewById(R.id.test_name)
        tableResult = view.findViewById(R.id.tableResult)

        //Mostramos el nombre del text
        tv_test_name.text = test[0].Name

        // Empezamos la funcion para mostrar las preguntas
        showQuestion(view)


        btn_next.setOnClickListener {
            //Almacenamos respuesta
            checkAnswer()

            //Log.d(TAG,answer_is_correct.toString())
            // Si la pregunta actual no es la última
            // Mostramos la siguiente pregunta
            if (current_question < all_questions.size - 1) {
                current_question++
                showQuestion(view)
            } else {
                // Si la pregunta actual es la última
                // Hacemos recuento de fallos y aciertos y lo mostramos por pantalla
                var correctas = 0.0
                var incorrectas = 0.0
                answer_is_correct.forEach {
                    if (it) correctas++
                    else incorrectas++
                }

                val div = correctas / (correctas + incorrectas)
                val valoracion: String = div.toString()

                // Actualizamos la valoración del Test en BD
                MainActivity.getDbHelper().updateTestVal(test[0].TestId, valoracion)

                showResult()
            }
        }


        btn_prev.setOnClickListener {
            //Alamacenamos respuesta
            checkAnswer()
            // Si la pregunta no es la primera
            // Mostramos la pregunta anterior
            if (current_question > 0) {
                current_question--
                showQuestion(view)
            }
        }

        return view
    }



    /**
     * Funcion que obtiene la repuesta marcada por el usuario, mira si es correcta y almacena la
     * respuesta marcada en user_anwswer y el resultado en answer_is_correct
     */

    private fun checkAnswer() {
        var id = rg_group.checkedRadioButtonId
        var answer = -1
        for (i in ids_answers.indices) {
            // Si este boton esta pulsado
            if (ids_answers[i] == id) answer = i
        }

        answer_is_correct[current_question] = (answer == correct_answer - 1)
        //Log.d(TAG,answer_is_correct.toString())
        user_answer[current_question] = answer
    }

    /**
     * Rellena el cuestionario de la pregunta actual con las posibles respuestas y obtiene la
     * respuesta correcta
     * @param view vista del fragment
     */
    private fun showQuestion(view: View) {

        // Recibimos las respuestas de la pregunta actual y las guardamos en un array
        answers = ArrayList()
        answers.add(all_questions[current_question].Answer1)
        answers.add(all_questions[current_question].Answer2)
        answers.add(all_questions[current_question].Answer3)
        answers.add(all_questions[current_question].Answer4)

        // Cambiamos el texto de la pregunta
        tv_test_question.text = all_questions[current_question].Text

        // Limpiamos el valor que habiamos maracado anteriormente
        rg_group.clearCheck()

        //Log.d(TAG,user_answer.toString())
        // Recorremos los diferentes botones para saber cual pulsa el usuario
        for (i in ids_answers.indices) {
            var rb = view.findViewById(ids_answers[i]) as RadioButton
            rb.text = answers[i]
            // Comprobamos si el usuario tenia una solucion marcada para esa pregunta
            if (user_answer[current_question] == i) {
                rb.isChecked = true
            }
        }

        //Log.d(TAG,String.format("%s",all_questions[current_question].toString()))

        // Obtenemos la respuesta correcta
        correct_answer = all_questions[current_question].CorrectAnswer.toInt()

        uiUpdate()

    }


    /**
     * Funcion que cambia la visibilidad de los botones Previo y Siguiente según la pregunta actual
     */
    private fun uiUpdate() {
        //Si es la primera cuestión no hay ninguna anterior
        if (current_question == 0) {
            btn_prev.visibility = View.GONE
        } else {
            btn_prev.visibility = View.VISIBLE
        }

        // Si el la última pregunta mostramos el botón de finalizar
        if (current_question == all_questions.size - 1) {
            btn_next.setText(R.string.finish)
        } else {
            btn_next.setText(R.string.next)
        }
    }

    /**
     * Función que muestra el resultado del test al usuario
     */
    private fun showResult() {

        val ivTam = 70

        // Para cada pregunta que tiene el tes vamos creando una tabla
        for(i in user_answer.indices){

            // Fila compuesta por un TextView y un ImageView
            val row = TableRow(context)
            val lpr = TableRow.LayoutParams(TableRow.LayoutParams.WRAP_CONTENT,TableRow.LayoutParams.WRAP_CONTENT)
            row.layoutParams = lpr

            val tvRespuesta = TextView(context)
            tvRespuesta.textSize = 30.0f
            val lptv = TableRow.LayoutParams(TableRow.LayoutParams.WRAP_CONTENT,TableRow.LayoutParams.WRAP_CONTENT)
            tvRespuesta.layoutParams = lptv

            val ivCorrecta = ImageView(context)
            val lpiv = TableRow.LayoutParams(ivTam,ivTam)
            ivCorrecta.layoutParams = lpiv

            // Comprobamos si la respuesta es correcta o no y cargamos una imagen acorde al resultado
            tvRespuesta.text = all_questions[i].Text
            if(answer_is_correct[i]==false){
                ivCorrecta.setImageResource(R.drawable.answer_bad)
            } else {
                ivCorrecta.setImageResource(R.drawable.answer_correct)
            }

            row.addView(ivCorrecta)
            row.addView(tvRespuesta)
            tableResult.addView(row,i)
        }


        // Mostramos la Tabla por pantalla
        rg_group.visibility = View.INVISIBLE
        tableResult.visibility = View.VISIBLE
        tv_test_question.visibility = View.INVISIBLE
        btn_prev.visibility = View.INVISIBLE
        btn_next.setOnClickListener({

            // Volvemos al fragment anterior en este caso InfoFragment
            fragmentManager!!.popBackStack()
        })
    }

}
